import java.util.*;

/* Un API (con Valore Astratto e specifica) per MUTABLE ParseTree<A,B>
   presentata il 25/5/2016
*/

class ParseTree<A, B extends A> implements ParseTreeAPI<A,B>{
    //Introduce Valori Astratti [ParseTree<A,B>], contenenti tutti e soli i valori [], per
    //l'albero vuoto, e [a-(t1,...tn)] per alberi non vuoti, aventi a\in[A] e
    //ti\in[ParseTree<A,B>] per tutti gli i\in[1..n].
    //Valori MUTABLE e senza vincoli di ampiezza e/o profondit\`a dell'albero.
    //Sia O un valore di tipo ParseTree<A,B> allora Va(O)\in[ParseTree<A,B>] \`e un cammino.
    //Stato Concreto
    private A label;
    private Vector<ParseTree<A,B>> sons;
    //AF e I
    //AF(c)=[] se c.label==null
    //AF(c)=[a]=[a-()] se c.label.equals(a) && c.sons.size()==0
    //AF(c)=[a-(t1,...,tn)] se n>=1 && AF(c.label)=a && c.sons.size()==n &&
    //   forall i\in[0..n-1], ti=AF(c.sons.get(i-1))
    //I(c)= c.sons===null sse c.label==null
    public ParseTree<A,B>(){
        label=null; sons=null;
    }
    public ParseTree<A,B>(A label){
        this.label = label;
        sons = new Vector<ParseTree<A,B>();
    }
    public ParseTree<A,B>(A label, LinkedList<ParseTree<A,B>> L){
        this.label = label;
        sons = new Vector<ParseTree<A,B>();
        for (ParseTree<A,B> u: L){
            sons.add(u);
        }
    }
    public boolean isEmpty(){
        // true sse Va(this)=[]
        return (label==null);
    }
    public boolean isLeaf(){
        // true sse Va(this)=[a-()]
        return (!(label==null)&& sons.size()==0);
    }
    public boolean isRoot(){
        // true sse Va(this)=[a-L] & L!=()
        return (sons!=null && sons.size()!=0);
    }
    public A getLabel() throws invalidOperationException{
        // Solleva invalidOperation se Va(this)=[]
        // Calcola a se Va(this)=[a-L] con L anche ()
        if (label==null) throw new invalidOperationException();
        return label;
    }
    public LinkedList<ParseTreeAPI<A,B>> getSons() throws invalidOperationException{
        // Solleva invalidOperation se Va(this)=[a-()] oppure Va(this)==[]
        // Calcola L se Va(this)=[a-L] con L!=()
        if (label==null || sons.size()==0) throw new invalidOperationException();
        LinkedList<Parsetree<A,B>> res = new LinkedList<ParseTree<A,B>>();
        for(ParseTree<A,B> u: sons){
            res.add(u);
        }
        return res;
    }
    public void set(Path p, ParseTreeAPI<A,B> t) throws invalidArgException{
        // Solleva invalidArgException se (p==null)||(t==null) o, p non \`e un cammino per this
        // Modifica this:
        //   this.post==this.pre se p=0
        //   this.post=[a-(t1,...,tn1-1,T,tn1+1,...,tk) se:
        //      (1) p=n1.q
        //      (2) this.pre=[a-(t1,...,tn1-1,tn1,tn1+1,...,tk)
        //      (3) tn1 \`e modificata in T dopo tn1.set(q,t).
        if (p==null || t==0) throw new invalidArgException();
        if (p.isEmpty()) return;
        int n1 = p.next();
        Path q = p.rest();
        if (n1>sons.size()) throw new invaldiArgException();
        sons.set(n1-1,((ParseTree<A,B>)sons.get(n1-1)).set(q,t));
    }
    //ADDITIONAL(equals,clone,elements,toString) omessa specifica eccetto che per:
    public LinkedList<ParseTree<A,B>> elements();
        // Calcola lista di tutti i sotto-alberi di this (this incluso)
}
