import java.util.*;

/* Un API (con Valore Astratto e specifica) per MUTABLE ParseTree<A,B>
   presentata il 25/5/2016
*/

interface ParseTreeAPI<A, B extends A> extends MuCloneable{
    //Introduce Valori Astratti [ParseTree<A,B>], contenenti tutti e soli i valori [], per
    //l'albero vuoto, e [a-(t1,...tn)] per alberi non vuoti, aventi a\in[A] e
    //ti\in[ParseTree<A,B>] per tutti gli i\in[1..n].
    //Valori MUTABLE e senza vincoli di ampiezza e/o profondit\`a dell'albero.
    //Sia O un valore di tipo ParseTree<A,B> allora Va(O)\in[ParseTree<A,B>] \`e un cammino.
    public boolean isEmpty();
        // true sse Va(this)=[]
    public boolean isLeaf();
        // true sse Va(this)=[a-()]
    public boolean isRoot();
        // true sse Va(this)=[a-L] & L!=()
    public A getLabel() throws invalidOperationException;
        // Solleva invalidOperation se Va(this)=[]
        // Calcola a se Va(this)=[a-L] con L anche ()
    public LinkedList<ParseTreeAPI<A,B>> getSons() throws invalidOperationException;
        // Solleva invalidOperation se Va(this)=[a-()] oppure Va(this)==[]
        // Calcola L se Va(this)=[a-L] con L!=()
    public void set(Path p, ParseTreeAPI<A,B> t) throws invalidArgException;
        // Solleva invalidArgException se (p==null)||(t==null) o, p non \`e un cammino per this
        // Modifica this:
        //   this.post==this.pre se p=0
        //   this.post=[a-(t1,...,tn1-1,T,tn1+1,...,tk) se:
        //      (1) p=n1.q
        //      (2) this.pre=[a-(t1,...,tn1-1,tn1,tn1+1,...,tk)
        //      (3) tn1 \`e modificata in T dopo tn1.set(q,t).
    //ADDITIONAL(equals,clone,elements,toString) omessa specifica eccetto che per:
    public LinkedList<ParseTree<A,B>> elements();
        // Calcola lista di tutti i sotto-alberi di this (this incluso)
}
